#! /usr/bin/env python
# coding: utf-8

import os
import time
import json
import sys
import logging
import argparse
from JYTools import jy_input, StringTool, logger
from JYTools.JYWorker import RedisStat, RedisQueue, DAGTools, TaskStatus

__author__ = '鹛桑够'


arg_man = argparse.ArgumentParser()
arg_man.add_argument("--debug", dest="debug", help="debug mode, print debug msg", action="store_true", default=False)


def parse_args():
    args = arg_man.parse_args()
    if args.debug is True:
        logger.setLevel(logging.DEBUG)
    return args


def empty_help():
    if len(sys.argv) <= 1:
        sys.argv.append("-h")


def list_queue():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="")
    args = parse_args()
    rs = RedisStat()
    if args.work_tag is None:
        qd = rs.list_queue()
        for key in qd:
            print(key)
    else:
        lqd = rs.list_queue_detail(args.work_tag)
        for item in lqd:
            print(item)


def list_worry_queue():
    rs = RedisStat()
    wq = rs.list_worry_queue()
    for item in wq:
        print(item)


def push_task():
    arg_man.add_argument("-c", "--config", dest="conf_path", help="configure file path")
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="WorkTag", required=True)
    arg_man.add_argument("-k", "--key", dest="key", help="task key", metavar="KEY", required=True)
    arg_man.add_argument("-s", "--sub-key", dest="sub_key", metavar="SubKey", help="task sub key")
    arg_man.add_argument("--is-report", dest="is_report", help="is report task", action="store_true", default=False)
    arg_man.add_argument("params", help="task params")
    args = parse_args()
    rq = RedisQueue(conf_path=args.conf_path, work_tag=args.work_tag)
    rq.push(args.key, args.params, sub_key=args.sub_key, is_report=args.is_report)


def report_task():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="WorkTag")
    arg_man.add_argument("-c", "--config", dest="conf_path", help="configure file path")
    arg_man.add_argument("-r", "--report-tag", dest="report_tag", help="work tag", metavar="ReportTag", required=True)
    arg_man.add_argument("-k", "--key", dest="key", help="task key", metavar="KEY", required=True)
    arg_man.add_argument("-s", "--sub-key", dest="sub_key", metavar="SubKey", help="task sub key")
    arg_man.add_argument("--start-time", dest="start_time", metavar="StartTime", help="start time. timestamp")
    arg_man.add_argument("--task-status", dest="task_status", metavar="TaskStatus",
                         choices=[TaskStatus.SUCCESS, TaskStatus.FAIL, TaskStatus.RUNNING], required=True)
    arg_man.add_argument("--task-output", dest="task_output", metavar="TaskOutput")
    arg_man.add_argument("message", help="task message")
    empty_help()
    args = parse_args()
    rq = RedisQueue(conf_path=args.conf_path, work_tag=args.report_tag)
    task_output = args.task_output
    if task_output is not None:
        task_output = json.loads(task_output)
    params = dict(end_time=int(time.time()), task_key=args.key, task_sub_key=args.sub_key, start_time=int(time.time()),
                  task_status=args.task_status, task_message=args.message, work_tag=args.work_tag,
                  task_output=task_output)
    if args.start_time is not None:
        params["start_time"] = args.start_time
    rq.push(args.key, params, sub_key=args.sub_key, is_report=True)


def list_heartbeat():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="")
    rs = RedisStat()
    args = parse_args()
    if args.work_tag is None:
        ws = rs.list_heartbeat()
        for item in ws:
            print(item)
    else:
        hd = rs.list_heartbeat_detail(args.work_tag)
        print(hd)


def delete_heartbeat():
    empty_help()
    rs = RedisStat()
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="", nargs="*", default=[])
    args = parse_args()
    for item in args.work_tag:
        print("delete heartbeat %s" % item)
        rs.delete_heartbeat(item)


def list_worker():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="")
    rs = RedisStat()
    args = parse_args()
    if args.work_tag is None:
        ws = rs.list_worker()
        for item in ws:
            print(item)
    else:
        hd = rs.list_worker_detail(args.work_tag)
        print(hd)


def auto_wash_worker_func(work_tag, redis_man=None):
    # 获取当前队列中是否任务
    # 获取当前多少个Worker
    rs = RedisStat(redis_man=redis_man)
    data = rs.list_worker_detail(work_tag)
    ids = data.keys()


def wash_worker():
    empty_help()
    arg_man.add_argument("-a", "--auto", dest="auto", help="auto set num", action="store_true", default=False)
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="", nargs="*", default=[])
    arg_man.add_argument("-n", "--num", dest="num", help="num of wash package to send", metavar="", type=int, default=1)
    args = parse_args()
    r_queue = RedisQueue()
    rs = RedisStat(redis_man=r_queue.redis_man)

    for item in args.work_tag:
        print("wash work tag %s" % item)
        if args.auto is True:
            data = rs.list_worker_detail(item)
            num = len(data.keys())
            print("In auto mode, send %s wash package %s" % (item, num))
            r_queue.wash_worker(item, num)
        else:
            r_queue.wash_worker(item, args.num)


def look_task_item():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="")
    args = parse_args()
    rs = RedisStat()
    values = []
    if args.work_tag is not None:
        values.append(args.work_tag)
    while True:
        prompt_prefix = ""
        if len(values) <= 0:
            work_tag = jy_input("Please Input Work_tag", prompt_prefix=prompt_prefix)
            work_tag = work_tag.strip()
            if work_tag.lower() in ("e", "exit"):
                sys.exit(0)
            values.append(work_tag)
            continue
        if len(values) == 1:
            prompt_prefix += "[work_tag:%s]" % values[0]
            key = jy_input("Please Input Task Key", prompt_prefix=prompt_prefix)
            key = key.strip()
            if key.lower() in ("e", "exit"):
                values.remove(values[-1])
                continue
            values.append(key)
            continue
        elif len(values) >= 2:
            sub_key = None
            prompt_prefix += "[work_tag:%s][key:%s]" % (values[0], values[1])
            if len(values) >= 3:
                sub_key = StringTool.join_decode(values[2:], "_")
                prompt_prefix += "[sub_key:%s]" % sub_key
            task_items = rs.list_task_item(values[0], values[1], sub_key)
            prompt = ""
            if len(task_items["sub"].keys()) > 0:
                prompt += "Input follow value look sub item\n"
                prompt += "\n".join(task_items["sub"])
                prompt += "\n"
            if len(task_items["values"].keys()) > 0:
                prompt += "Input follow value look item value\n"
                prompt += "\n".join(task_items["values"])
            while True:
                item_key = jy_input(prompt, prompt_prefix=prompt_prefix)
                item_key = item_key.strip()
                if item_key.lower() in ("e", "exit"):
                    values.remove(values[-1])
                    break
                if item_key in task_items["sub"]:
                    values.append(item_key)
                    break
                elif item_key in task_items["values"]:
                    print(task_items["values"][item_key])
                    continue
                else:
                    continue


def clear_dirty_item():
    arg_man.add_argument("-w", "--work-tag", dest="work_tag", help="work tag", metavar="")
    empty_help()
    args = parse_args()
    rs = RedisStat()
    work_tag = args.work_tag
    dirty_items = rs.get_dirty_item(work_tag=args.work_tag)
    if len(dirty_items) == 0:
        print("Not Found")
    for item in dirty_items:
        message = "Are You Sure Delete %s, Include [%s]. Message Is %s" % (item["prefix"], ",".join(item["sub_keys"]),
                                                                           item["message"])
        message += "\nInput Y/n  "
        sure = jy_input(message)
        if sure.lower() == "y":
            print("Delete %s" % item["prefix"])
            rs.clear_task_item(work_tag, item["prefix"])


def verify_pipeline():
    arg_man.add_argument("file", help="pipeline file", metavar="file")
    empty_help()
    args = parse_args()
    p_file = args.file
    if os.path.isfile(p_file) is False:
        logger.error("file %s not exist" % p_file)
        sys.exit(1)
    with open(p_file) as rp:
        c = rp.read()
    try:
        c_o = json.loads(c)
    except ValueError:
        logger.error("The content of the file is not legal, not the json content")
        sys.exit(1)
    r, data = DAGTools.ip_verify_pipeline(c_o)
    sys.exit(data["code"])


def clear_worker():
    rs = RedisStat()
    r_queue = RedisQueue(redis_man=rs.redis_man)
    ws = rs.list_worker()
    for item in ws:
        rs.delete_heartbeat(item)
        data = rs.list_worker_detail(item)
        num = len(data.keys())
        r_queue.wash_worker(item, num)



if __name__ == "__main__":
    # sys.argv.append("--debug")
    sys.argv.extend(["-w", "Pipeline"])
    # wash_worker()
    # sys.argv.extend(["--report-tag", "Pipeline", "-k", "100", "-s", "2", "--task-output", '{"a":"a"}', "--task-status", "Success", "success"])
    print(" ".join(sys.argv))
    # list_queue()
    clear_dirty_item()


# !/usr/bin/env python
# coding: utf-8
import json
import uuid

from JYTools.JYWorker import RedisQueue
from JYTools.JYWorker import RedisWorker
from JYTools.util import loader

from JYTools.util import constants


__author__ = 'zhouhenglc'


def get_func_name(func):
    if func is None:
        return
    if not callable(func):
        raise RuntimeError('function %s not callable' % func)
    func_name = '%s.%s' % (func.__module__, func.__name__)
    return func_name


def handle_one_failed_func(failed_func, **kwargs):
    status = kwargs['status']
    if status != TASK_STATUS_FAILED:
        return
    f_func = loader.load_function(failed_func)
    if not f_func:
        return
    f_func(kwargs)


def handle_completed_func(completed_func, success_func, failed_func,
                          *result, **kwargs):
    if completed_func:
        c_func = loader.load_function(completed_func)
        c_func(result)
    for r in result:
        if r['status'] != TASK_STATUS_SUCCESS:
            if not failed_func:
                return
            f_func = loader.load_function(failed_func)
            f_func(result)
            return
    s_func = loader.load_function(success_func)
    s_func(result)
    return


TASK_STATUS_SUCCESS = 'success'
TASK_STATUS_FAILED = 'failed'


class FuncTaskWorker(RedisWorker):
    expect_params_type = dict
    DEFAULT_WORK_TAG = constants.DEFAULT_FUN_TASK_TAG

    def handle_task(self, key, params):
        if 'func' not in params:
            self.set_current_task_invalid('Need func')
        exec_args = params.get('exec_args', [])
        exec_kwargs = params.get('exec_kwargs', {})
        reload = params.get('reload', False)
        exception = None
        _func_name = params['func']
        self.task_log('exec args is: ', *exec_args)
        try:
            _func = loader.load_function(_func_name, reload)
            r = _func(*exec_args, **exec_kwargs)
            status = TASK_STATUS_SUCCESS
        except Exception as e:
            status = TASK_STATUS_FAILED
            self.task_exception_log()
            exception = {'msg': str(e), 'type': e.__class__.__name__}
            r = None
        exec_result = {'status': status, 'exception': exception,
                       'func_return': r, 'exec_args': exec_args,
                       'exec_kwargs': exec_kwargs}
        self.set_output("exec_result", exec_result)


class FuncTaskGroup(object):

    def __init__(self, group_id, completed_func=None, failed_func=None,
                 success_func=None, any_failed_func=None, func_task_tag=None,
                 pipeline_tag=None, redis_host=None, redis_password=None,
                 redis_port=None, redis_db=None, redis_man=None):
        if group_id is None:
            group_id = 'atg-%s' % str(uuid.uuid4())
        self.group_id = group_id
        self.func_task_tag = func_task_tag or constants.DEFAULT_FUN_TASK_TAG
        self.pipeline_tag = pipeline_tag or constants.DEFAULT_PIPELINE_TAG
        self._internal_id = 'iatg-%s' % str(uuid.uuid4())
        self.completed_func = get_func_name(completed_func)
        self.failed_func = get_func_name(failed_func)
        self.success_func = get_func_name(success_func)
        self.any_failed_func = get_func_name(any_failed_func)
        self.tasks = []
        self.started = False
        queue_kwargs = {'redis_host': redis_host, 'redis_port': redis_port,
                        'redis_password': redis_password,
                        'redis_db': redis_db, 'redis_man': redis_man,
                        'work_tag': self.pipeline_tag}
        self.r_queue = RedisQueue(**queue_kwargs)

    def add_sync_task(self, func, *args, **kwargs):
        func_name = get_func_name(func)
        params = {'work_tag':  self.func_task_tag,
                  'input_exec_args': list(args),
                  'input_exec_kwargs': kwargs,
                  'input_func': func_name}

        self.tasks.append(params)

    def start(self):
        if self.started:
            return
        self.started = True
        task_list = self.tasks
        task_len = len(self.tasks)
        if self.any_failed_func:
            for i in range(task_len):
                task = {'work_tag':  self.func_task_tag,
                        'input_func': get_func_name(handle_one_failed_func),
                        'input_exec_args': [self.any_failed_func],
                        'input_exec_kwargs': '&%sexec_result' % (i+1)}
                task_list.append(task)
        c_task = {'work_tag':  self.func_task_tag,
                  'input_func': get_func_name(handle_completed_func),
                  'input_exec_args': [self.completed_func, self.success_func,
                                      self.failed_func],
                  'input_exec_kwargs': {}}
        c_task['input_exec_args'].extend(
            ['&%sexec_result' % (i+1) for i in range(task_len)])
        task_list.append(c_task)
        key = '%s-%s' % (self.group_id, self._internal_id)
        params = {'task_list': task_list}
        self.r_queue.push(key, params=params)


def run_fun_task(redis_queue, key, func, *args, **kwargs):
    work_tag = redis_queue.work_tag or constants.DEFAULT_FUN_TASK_TAG
    params = {'exec_args': list(args), 'exec_kwargs': kwargs,
              'func': get_func_name(func), 'reload': False}
    redis_queue.push(key, params, work_tag=work_tag)



# !/usr/bin/env python
# coding: utf-8
import importlib

__author__ = 'zhouhenglc'


def load_function(func_name, reload=False):
    """

    :param func_name:  mode_name.func_name
            eg: JYTools.demo.some_funcs.func_plus
    :param reload:
    :return:
    """
    mod_name, _f_name = func_name.rsplit('.', 1)
    mod = importlib.import_module(mod_name)
    if reload:
        importlib.reload(mod)
    func = getattr(mod, _f_name)
    return func


if __name__ == '__main__':
    func = load_function('JYTools.demo.some_funcs.func_plus', reload=True)



#! /usr/bin/env python
# coding: utf-8

import os
from email.header import Header
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
import smtplib
from multiprocessing.pool import ThreadPool
import configparser

__author__ = 'ZhouHeng'


class EmailManager(object):
    def __init__(self, conf_dir, thread_num=5):
        self.email_server = "ym.163.com"
        self.sender = "晶云平台"
        conf_path = os.path.join(conf_dir, "email_app.conf")
        if os.path.exists(conf_path) is False:
            conf_path = os.path.join(conf_dir, "email.conf")
        self.m_user = "admin@localhost"
        self.m_password = ""
        self._int_app(conf_path)
        self.encoding = 'utf-8'
        self.send_user = '%s <%s>' % (Header(self.sender, self.encoding), self.encoded(self.m_user))
        self.t_pool = ThreadPool(thread_num)

    def _int_app(self, conf_path):
        config = configparser.ConfigParser()
        config.read(conf_path)
        section = "admin"
        try:
            self.m_user = config.get(section, "email")
            self.m_password = config.get(section, "password")
            self._send = self._remote_send
        except configparser.Error:
            self._send = self._local_send

    def encoded(self, s, encoding="utf-8"):
        return s.encode(encoding) if isinstance(s, str) else s

    def _remote_send(self, to, sub, msg):
        try:
            SMTP = smtplib.SMTP_SSL
            smtp = SMTP("smtp.%s" % self.email_server, 465)
            # smtp.set_debuglevel(True)
            smtp.login(self.m_user, self.m_password)

            msg['From'] = self.send_user
            msg['To'] = self.encoded(to, self.encoding)
            msg['Subject'] = Header(self.encoded(sub, self.encoding), self.encoding)

            smtp.sendmail(self.m_user, to, msg.as_string())
            smtp.quit()
            return True
        except Exception as e:
            error_message = "MyEmailManager send_mail error %s" % str(e)
            print(error_message)
            return False

    def _local_send(self, to, sub, msg):
        try:
            smtp = smtplib.SMTP("localhost")
            # smtp.set_debuglevel(True)
            msg['From'] = self.send_user
            msg['To'] = self.encoded(to, self.encoding)
            msg['Subject'] = Header(self.encoded(sub, self.encoding), self.encoding)

            smtp.sendmail(self.m_user, to, msg.as_string())
            smtp.quit()
            return True
        except Exception as e:
            error_message = "MyEmailManager send_mail error %s" % str(e)
            print(error_message)
            return False

    def send_mail(self, to, sub, content):
        return self.send_attachment(to, sub, content, [])

    def send_attachment(self, to, sub, content, attachments):
        msg = MIMEMultipart('alternative')
        msg.attach(MIMEText(self.encoded(content, self.encoding), "html", self.encoding))
        if isinstance(attachments, list):
            real_attachments = set(map(lambda x: os.path.realpath(x), attachments))
            for attach in real_attachments:
                att_item = MIMEText(open(attach, "rb").read(), 'base64', self.encoding)
                att_item["Content-Type"] = "application/octet-stream"
                att_item["Content-Disposition"] = "attachment; filename=%s" % os.path.basename(attach)
                msg.attach(att_item)
        return self._send(to, sub, msg)

    def send_mail_thread(self, to, sub, content):
        return self.t_pool.apply_async(self.send_mail, (to, sub, content))

    def send_attachment_thread(self, to, sub, content, attachments):
        return self.t_pool.apply_async(self.send_attachment, (to, sub, content, attachments))

if __name__ == "__main__":
    email_man = EmailManager(conf_dir="/mnt/data/JINGD/conf")
    # email_man.send_mail("zhouheng@gene.ac", "Final TEST", "TEST NEW SEND. Only send content")
    # email_man.send_attachment("zhou5315938@163.com", "Final Test", "TEST send Fom local", ["/home/msg/a.txt", "/home/msg/a.txt", "/home/msg/a.txt"])
    # email_man.send_attachment_thread("zhouheng@gene.ac", "Final Test", "TEST send Fom local", ["/home/msg/a.txt", "/home/msg/a.txt", "/home/msg/a.txt"])
    email_man.send_mail_thread("zhouheng@gene.ac", "邮箱验证码", open("/home/msg/tmpmail_21.html").read())
    email_man.t_pool.close()
    email_man.t_pool.join()



#! /usr/bin/env python
# coding: utf-8

from JYTools import StringTool

__author__ = '鹛桑够'


class FileWriter(object):

    def __init__(self, file_path, mode="w"):
        self.file_path = StringTool.encode(file_path)
        self.mode = mode
        self._w = None
        self._w = open(self.file_path, mode=self.mode)

    def __enter__(self):
        return self

    def write(self, s):
        self._w.write(s)

    def write_line(self, s):
        self._w.write(s + "\n")

    def write_array(self, a, connector="\t"):
        s = StringTool.join_encode(a, join_str=connector)
        self.write_line(s)

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._w.close()

if __name__ == "__main__":
    with FileWriter("test鹛桑够.txt") as fw:
        fw.write("this 鹛桑够")
        fw.write(u"this 鹛桑够")
        fw.write_line("this 鹛桑够")
        fw.write_array(["this", "鹛桑够", u"鹛桑够"])
    fw2 = FileWriter(u"test鹛桑够2.txt")
    fw2.write("this 鹛桑够")
    fw2.write(u"this 鹛桑够")
    fw2.write_line("this 鹛桑够")
    fw2.write_array(["this", "鹛桑够", u"鹛桑够", 1])


#! /usr/bin/env python
# coding: utf-8

from JYTools.JYWorker import DAGWorker

__author__ = 'meisanggou'


def main():
    DAGWorker.init_parser.add_argument("-a", "--agent-tag", dest="agent_tag", help="agent tag")
    DAGWorker.other_parser.add_argument("-m", "--mns-conf-path", dest="mns_conf_path", help="mns conf path")
    args = DAGWorker.parse_args()
    if args.work_tag is None:
        args.work_tag = "Pipeline"
    # args.agent_tag = "PBSAgent"
    app = DAGWorker(conf_path=args.conf_path, heartbeat_value=args.heartbeat_value, work_tag=args.work_tag,
                    log_dir=args.log_dir, agent_tag=args.agent_tag)
    try:
        from JYAliYun.AliYunAccount import RAMAccount
        from JYAliYun.AliYunMNS.AliMNSServer import MNSServerManager
        if args.mns_conf_path is not None:
            mns_account = RAMAccount(conf_path=args.mns_conf_path)
            mns_server = MNSServerManager(mns_account, conf_path=args.mns_conf_path)
            mns_topic = mns_server.get_topic("JYWaring")
            app.msg_manager = mns_topic
    except Exception as e:
        print(e)
    app.work(daemon=args.daemon)

if __name__ == "__main__":
    from JYTools.JYWorker import worker_run
    worker_run(DAGWorker)



#! /usr/bin/env python
# coding: utf-8

import random
import syslog
from JYTools.util.string_rich import StringData
from redis import Redis, RedisError

__author__ = '鹛桑够'


class StrongRedis(Redis):

    def execute_command(self, *args, **options):
        try:
            return Redis.execute_command(self, *args, **options)
        except RedisError as r_error:
            syslog.syslog(*args)
            syslog.syslog(r_error)


class CacheRedis(Redis):

    def __init__(self, key_prefix=None, error_sleep=1000, host='localhost', port=6379, db=0, password=None,
                 socket_connect_timeout=1):
        self.key_prefix = key_prefix
        if key_prefix is not None:
            Redis.__init__(self, host, port, db, password=password, socket_connect_timeout=socket_connect_timeout)
        self.error_num = 0
        self.error_sleep = error_sleep

    def execute_command(self, *args, **options):
        if 0 < self.error_num < self.error_sleep:
            self.error_num += 1
            return None
        try:
            return Redis.execute_command(self, *args, **options)
        except RedisError as r_error:
            self.error_num = 1
            syslog.syslog(str(r_error))
            return None

    def get(self, name):
        if self.key_prefix is None:
            return None
        name = self.key_prefix + name
        v = Redis.get(self, name)
        v = StringData.unpack_data(v)
        return v

    def set(self, name, value, ex=None, px=None, nx=False, xx=False):
        if self.key_prefix is None:
            return None
        value = StringData.package_data(value)
        name = self.key_prefix + name
        return Redis.set(self, name, value, ex=ex, px=px, nx=nx, xx=xx)

    def setnx(self, name, value):
        if self.key_prefix is None:
            return None
        name = self.key_prefix + name
        value = StringData.package_data(value)
        return Redis.setnx(self, name, value)

    def setex(self, name, value, time):
        if self.key_prefix is None:
            return None
        value = StringData.package_data(value)
        name = self.key_prefix + name
        return Redis.setex(self, name, value, time)

    def setex2(self, name, value, basic_time=60, random_ranges=None):
        if random_ranges is None:
            random_ranges = basic_time
        time = basic_time + random.randint(0, random_ranges)
        return self.setex(name, value, time)

    def delete(self, *names):
        if self.key_prefix is None:
            return None
        names = map(lambda x: self.key_prefix + x, names)
        return Redis.delete(self, *names)

if __name__ == "__main__":
    redis_man = CacheRedis("jy_api", host="172.16.110.5", port=9532, db=1)
    print(redis_man.set("bac", dict(s="123")))
    print(redis_man.get("bac"))
    redis_man.delete("bac")



#! /usr/bin/env python
# coding: utf-8

from time import sleep
from JYTools.StringTool import is_string
from ._redis import RedisWorker

__author__ = 'meisanggou'


class AsyncRedisWorker(RedisWorker):

    def __init__(self, conf_path=None, heartbeat_value=None, work_tag=None, log_dir=None, stat_work_tag=None, **kwargs):
        RedisWorker.__init__(self, conf_path=conf_path, heartbeat_value=heartbeat_value, work_tag=work_tag,
                             log_dir=log_dir, **kwargs)
        if is_string(stat_work_tag) is False:
            raise TypeError("stat work tag must be string")
        self.stat_work_tag = stat_work_tag
        self.after_handler_funcs.append(self.after_handler)

    def after_handler(self):
        r_tag = self.current_task.task_report_tag
        self.current_task.task_report_tag = None
        self.push_task(self.current_task.task_key, self.current_task.task_output, work_tag=self.stat_work_tag,
                       report_tag=r_tag, sub_key=self.current_task.task_sub_key)


class AsyncStatRedisWorker(RedisWorker):

    def __init__(self, conf_path=None, heartbeat_value=None, work_tag=None, log_dir=None, **kwargs):
        RedisWorker.__init__(self, conf_path=conf_path, heartbeat_value=heartbeat_value, work_tag=work_tag,
                             log_dir=log_dir, **kwargs)
        self.top_queue = None
        self.before_handler_funcs.append(self.before_handler)
        self.sleep_time = 10

    def before_handler(self):
        if self.current_task == self.top_queue:
            self.task_log("start sleep", self.sleep_time, "seconds")
            sleep(self.sleep_time)
            self.top_queue = None

    def whether_completed(self, key, params):
        return True

    def handler_task(self, key, params):
        completed = self.whether_completed(key, params)
        self.task_log("Async Task Stat Is", completed)
        if completed is True:
            self.set_multi_output(**params)
            return
        r_tag = self.current_task.task_report_tag
        self.current_task.task_report_tag = None
        self.push_task(self.current_task.task_key, params, report_tag=r_tag,
                       sub_key=self.current_task.task_sub_key)
        if self.top_queue is None:
            self.top_queue = self.current_task



#! /usr/bin/env python
# coding: utf-8

import os
import sys
import logging
import types
from ._redis import RedisWorker, RedisQueue, RedisStat
from ._Worker import ReadWorkerLog
from ._async import AsyncRedisWorker, AsyncStatRedisWorker
from ._Task import TaskStatus, TaskType
from ._du import DAGWorker, DAGTools
from .UploadLog import UploadLogWorker

__author__ = 'meisanggou'


logging.basicConfig(level=logging.INFO, format="%(message)s")


def _print():
    print()


def receive_argv(d, short_opt, long_opt, default_value="", **kwargs):
    short_opt_key = "-" + short_opt
    long_opt_key = "--" + long_opt
    if short_opt_key in d:
        print("use %s %s" % (short_opt_key, d[short_opt_key]))
        return d[short_opt_key]
    if long_opt_key in d:
        print("use %s %s" % (long_opt_key, d[long_opt_key]))
        return d[long_opt_key]
    env_key = long_opt.replace("-", "_").upper()
    env_value = os.environ.get(long_opt.replace("-", "_").upper())
    if env_value is not None:
        print("use env value %s: %s" % (env_key, env_value))
        return env_value
    if default_value == "":
        exit("please use %s or %s" % (short_opt_key, long_opt_key))
    return default_value


def _sub_class(b_class, c_class):
    """
    add in 1.0.1
    :param b_class:
    :param c_class:
    :return:
    """
    if issubclass(b_class, c_class) is True:
        return b_class
    if issubclass(c_class, b_class) is True:
        return c_class
    return None


def min_class(classes):
    """
    add in 1.0.1
    :param classes:
    :return:
    """
    if isinstance(classes, list) is False:
        return []
    right_classes = []
    for c_item in classes:
        if isinstance(c_item, types.TypeType) is False:
            continue
        if issubclass(c_item, RedisWorker):
            right_classes.append(c_item)
    if len(right_classes) <= 0:
        return []

    m_s = [right_classes[0]]
    for i in range(1, len(right_classes)):
        for j in range(len(m_s)):
            t_class = _sub_class(m_s[j], right_classes[i])
            if t_class is None:
                m_s.append(right_classes[i])
            else:
                m_s[j] = t_class
    return m_s


def worker_run(worker_class, default_work_tag=None):
    if issubclass(worker_class, RedisWorker) is False:
        print("Error worker class")
        return 1, None
    if issubclass(worker_class, RedisWorker) is True:
        args = worker_class.parse_args()
        if args.sys_paths:
            for path in args.sys_paths:
                sys.path.append(path)
        if args.debug is True:
            logging.basicConfig(level=logging.DEBUG)
        if args.work_tag is None:
            args.work_tag = default_work_tag
        app = worker_class(conf_path=args.conf_path, heartbeat_value=args.heartbeat_value, work_tag=args.work_tag,
                           log_dir=args.log_dir)
        if args.example_path is not None:
            o = app.test(key=args.key, params_path=args.example_path, sub_key=args.sub_key, report_tag=args.report_tag,
                         report_scene=args.report_scene)
            return 0, o
        else:
            app.work(args.daemon)
    return 0, None
